"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.call = void 0;
const mz_1 = require("mz");
const path = require("path");
const Raven = require("raven");
const rimraf = require("rimraf");
const find_dependency_dependencies_1 = require("./dependencies/find-dependency-dependencies");
const install_dependencies_1 = require("./dependencies/install-dependencies");
const find_package_infos_1 = require("./packages/find-package-infos");
const find_requires_1 = require("./packages/find-requires");
const get_hash_1 = require("./utils/get-hash");
function getContents(dependency, packagePath, packageInfos) {
    return __awaiter(this, void 0, void 0, function* () {
        const contents = yield find_requires_1.default(dependency, packagePath, packageInfos);
        const packageJSONFiles = Object.keys(packageInfos).reduce((total, next) => (Object.assign(Object.assign({}, total), { [next.replace(packagePath, "")]: {
                content: JSON.stringify(packageInfos[next]),
            } })), {});
        // // Hardcoded deletion of some modules that are not used but added by accident
        // deleteHardcodedRequires(
        //   contents,
        //   "/node_modules/react/cjs/react.production.min.js",
        // );
        // deleteHardcodedRequires(
        //   contents,
        //   "/node_modules/react-dom/cjs/react-dom.production.min.js",
        // );
        return Object.assign(Object.assign({}, contents), packageJSONFiles);
    });
}
/**
 * Delete `module` field if the module doesn't exist at all
 */
function verifyModuleField(pkg, pkgLoc) {
    if (!pkg.module) {
        return;
    }
    try {
        const basedir = path.dirname(pkgLoc);
        const found = [
            path.join(basedir, pkg.module),
            path.join(basedir, pkg.module, "index.js"),
            path.join(basedir, pkg.module, "index.mjs"),
        ].find((p) => {
            try {
                const l = mz_1.fs.statSync(p);
                return l.isFile();
            }
            catch (e) {
                return false;
            }
        });
        if (!found) {
            pkg.csbInvalidModule = pkg.module;
            delete pkg.module;
        }
    }
    catch (e) {
        /* */
    }
}
let packaging = false;
function call(event, context, cb) {
    return __awaiter(this, void 0, void 0, function* () {
        /** Immediate response for WarmUP plugin */
        if (event.source === "serverless-plugin-warmup") {
            console.log("WarmUP - Lambda is warm!");
            return cb(undefined, "Lambda is warm!");
        }
        const dependency = event;
        const hash = get_hash_1.default(dependency);
        const t = Date.now();
        if (!hash) {
            return;
        }
        if (!dependency) {
            return;
        }
        const packagePath = path.join("/tmp", hash);
        // Cleanup!
        if (!packaging) {
            console.log("Cleaning up /tmp");
            try {
                const folders = mz_1.fs.readdirSync("/tmp");
                folders.forEach((f) => {
                    const p = path.join("/tmp/", f);
                    try {
                        if (mz_1.fs.statSync(p).isDirectory() && p !== "/tmp/git") {
                            rimraf.sync(p);
                        }
                    }
                    catch (e) {
                        console.error("Could not delete " + p + ", " + e.message);
                    }
                });
            }
            catch (e) {
                console.error("Could not delete dependencies: " + e.message);
                console.log("Continuing packaging...");
            }
        }
        packaging = true;
        try {
            const finalVersion = yield install_dependencies_1.default(dependency, packagePath);
            const packageInfos = yield find_package_infos_1.default(dependency.name, packagePath);
            Object.keys(packageInfos).map((pkgJSONPath) => {
                const pkg = packageInfos[pkgJSONPath];
                verifyModuleField(pkg, pkgJSONPath);
            });
            const contents = yield getContents(dependency, packagePath, packageInfos);
            console.log("Done - " +
                (Date.now() - t) +
                " - " +
                dependency.name +
                "@" +
                dependency.version);
            const requireStatements = new Set();
            Object.keys(contents).forEach((p) => {
                const c = contents[p];
                if (c.requires) {
                    c.requires.forEach((r) => requireStatements.add(r));
                }
            });
            const dependencyDependencies = find_dependency_dependencies_1.default(dependency, packagePath, packageInfos, requireStatements, contents);
            const response = {
                source: Object.assign({ contents,
                    dependency }, dependencyDependencies),
                version: finalVersion
            };
            // Cleanup
            try {
                rimraf.sync(packagePath);
            }
            catch (e) {
                /* ignore */
            }
            cb(undefined, response);
        }
        catch (e) {
            // Cleanup
            try {
                rimraf.sync(packagePath);
            }
            catch (e) {
                /* ignore */
            }
            console.error("ERROR", e);
            Raven.captureException(e, {
                tags: {
                    hash,
                    dependency: `${dependency.name}@${dependency.version}`,
                },
            });
            cb(undefined, { error: e.message });
        }
        finally {
            packaging = false;
        }
    });
}
exports.call = call;
// call({
//   name: "@jd/tools",
//   version: "latest",
//   nodePath: null,
// }, {} as any, (err, result) => {
//   console.log(err);
//   console.log('====>', result.version);
//   // s3.saveResult(result);
//   // console.log(result);
// });
exports.default = call;
//# sourceMappingURL=index.js.map