"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const mz_1 = require("mz");
const path_1 = require("path");
const BLACKLISTED_DIRS = [
    "demo",
    "docs",
    "benchmark",
    "flow-typed",
    "src",
    "bundles",
    "examples",
    "scripts",
    "tests",
    "test",
    "umd",
    "min",
    "node_modules",
];
function getFilePathsInDirectory(path) {
    return __awaiter(this, void 0, void 0, function* () {
        const entries = yield mz_1.fs.readdir(path);
        const entriesWithMetadata = yield Promise.all(entries
            .map((fPath) => path_1.join(path, fPath))
            .map((entry) => __awaiter(this, void 0, void 0, function* () {
            const meta = yield mz_1.fs.lstat(entry);
            return { entry, isDirectory: meta.isDirectory() };
        })));
        let files = entriesWithMetadata
            .filter((x) => !x.isDirectory)
            .map((x) => x.entry);
        const childFiles = yield Promise.all(entriesWithMetadata
            .filter((x) => x.isDirectory)
            .map((x) => x.entry)
            .filter((x) => BLACKLISTED_DIRS.indexOf(path_1.basename(x)) === -1)
            .filter((x) => !path_1.basename(x).startsWith("."))
            .map((dir) => getFilePathsInDirectory(dir)));
        childFiles.forEach((f) => {
            files = [...files, ...f];
        });
        return files;
    });
}
const DISALLOWED_EXTENSIONS = ["min.js", "umd.js", "node.js", "test.js"];
const ALLOWED_EXTENSIONS = [
    "json",
    "js",
    "css",
    "scss",
    "styl",
    "less",
    "vue",
    "html",
];
function isValidFile(packagePath, packageInfo) {
    return (filePath) => {
        const relDirName = filePath.replace(packagePath, "").slice(1);
        if (path_1.basename(filePath).startsWith(".")) {
            return false;
        }
        if (BLACKLISTED_DIRS.some((dir) => {
            return relDirName.startsWith(dir);
        })) {
            return false;
        }
        if (DISALLOWED_EXTENSIONS.some((ex) => filePath.endsWith(ex))) {
            return false;
        }
        if (ALLOWED_EXTENSIONS.some((ex) => filePath.endsWith(ex))) {
            return true;
        }
        return false;
    };
}
const FALLBACK_DIRS = ["dist", "lib", "build"];
function resolveRequiredFiles(packagePath, packageInfo) {
    return __awaiter(this, void 0, void 0, function* () {
        let main = typeof packageInfo.browser === "string"
            ? packageInfo.browser
            : packageInfo.module || packageInfo.main;
        let entryDir;
        if (!main) {
            const indexFileExists = mz_1.fs.existsSync(path_1.join(packagePath, "index.js"));
            if (indexFileExists) {
                main = "index.js";
                entryDir = packagePath;
            }
            else {
                entryDir = FALLBACK_DIRS.map((d) => path_1.join(packagePath, d)).find((dir) => mz_1.fs.existsSync(dir) && mz_1.fs.lstatSync(dir).isDirectory());
            }
        }
        else {
            entryDir = path_1.join(packagePath, path_1.dirname(main));
        }
        if (!entryDir) {
            return [];
        }
        const browser = typeof packageInfo.browser === "object" ? packageInfo.browser : {};
        const browserAliases = Object.keys(browser).reduce((total, path) => {
            const relativePath = path_1.join(packagePath, path);
            let resolvedPath = browser[path];
            if (resolvedPath !== false) {
                resolvedPath = path_1.join(packagePath, resolvedPath);
            }
            return Object.assign(Object.assign({}, total), { [relativePath]: resolvedPath });
        }, {});
        const isValidFileTest = isValidFile(entryDir, packageInfo);
        // I removed this optimization Our browser and caching strategy is nowadays so sophisticated that
        // this only introduces unnecessary bagage.
        const files = (yield getFilePathsInDirectory(entryDir))
            .filter(isValidFileTest)
            .map((path) => {
            if (typeof browserAliases === "object") {
                if (browserAliases[path] === false) {
                    return null;
                }
                if (browserAliases[path]) {
                    return browserAliases[path];
                }
            }
            return path;
        })
            .filter((x) => x != null);
        if (main) {
            [path_1.join(packagePath, main), path_1.join(packagePath, main, "index.js")].find((p) => {
                try {
                    const stat = mz_1.fs.statSync(p);
                    if (stat.isFile()) {
                        files.push(p);
                        return true;
                    }
                    return false;
                }
                catch (e) {
                    return false;
                }
            });
        }
        return files;
    });
}
exports.default = resolveRequiredFiles;
//# sourceMappingURL=resolve-required-files.js.map